/*
     []-------------------------------------------------------------[]
     |		                 INTER.C                             |
     |						 	             |
     |      Determina a intersecao entre duas superficies usando     |
     |            uma tecnica mista subdivisao + caminhada           |
     |                                                               |
     |             por Lenimar N. Andrade, maio/96-maio/98           |
     []-------------------------------------------------------------[]
*/

/* #define EXEMPLO16 */

#include "basico.c"
#include "equacoes.c"
#include "converg.c"
#include "subdiv.c"
#include "str.c"
#include "isolado.c"

#define SENTIDO_VETOR_TANGENTE 0
#define SENTIDO_CONTRARIO      1

#ifdef BEZIER
#include "bezier.c"
#endif

int main(int argc, char *argv[]) {

  ponto_dominio p0, q0, p0inv, q0inv;
  ponto P, P1, P2, P_temp, Pinv, P1inv, P2inv;
  vetor T, T1, T2, T_temp, Tinv, T1inv, T2inv;
  long n_subdiv, n_pts_selec1 = 0, n_pts_selec2 = 0;
  char linha_div1[100], linha_div2[100], pt_atual[150];
  float dist_div;
  int n_pt = 0, iteracoes, outro_novo = 1;
  unsigned long total_pontos = 0;

  #ifndef UNIX
  struct time inicio, fim;
  double tempo_gasto;
  #endif

  #include "ajustes.c"

  r_erro = PASSO_L*2.0/3.0;

  /* ---------------------------- Inicio --------------------------------- */
  remove(NOME_ARQ_ERROS);
  NomeArquivoCfg(Minusculas(argv[0]), "cfg", ARQ_CONFIGURACAO);

  if (LeConfiguracao(ARQ_CONFIGURACAO)) {
    if (SALVA_HIST) {
      if ((arq_execucao = fopen(ARQ_EXECUCAO, "wt")) == NULL)
	SALVA_HIST = 0;
    }
    if (MOSTRA_HIST)
      printf("\n# Configuracao definida em %s", ARQ_CONFIGURACAO);
    if (SALVA_HIST)
      fprintf(arq_execucao, "\n# Configuracao definida em %s", ARQ_CONFIGURACAO);
  } else if (SALVA_HIST) {
      if ((arq_execucao = fopen(ARQ_EXECUCAO, "wt")) == NULL)
	SALVA_HIST = 0;
  }

  if ((arq_padrao = fopen(ARQ_DADOS_PADRAO, "wt")) == NULL) {
    Erro("Nao foi possivel abrir o arquivo de dados");
    return 1;
  }

  /* -------------------------- Subdivisao ------------------------------- */

  if (argc > 1) {
    strcpy(DIV_CLASS1, argv[1]);
    n_pts_selec1 = ContaPontoSubdivDominio(DIV_CLASS1);
    if (DOMINIO2 && argc > 2) {
      strcpy(DIV_CLASS2, argv[2]);
      n_pts_selec2 = ContaPontoSubdivDominio(DIV_CLASS2);
    } else n_pts_selec2 = 0;
  } else {

  #ifndef UNIX
  HoraAtual(&inicio);      /* Inicio da subdivisao */
  #endif

  if (MOSTRA_HIST)
    printf("\n\n# Subdivisao dos dominios ate' nivel %d", NIVEL_SUBDIV);
  if (SALVA_HIST)
    fprintf(arq_execucao, "\n\n# Subdivisao dos dominios ate' nivel %d", NIVEL_SUBDIV);
  n_subdiv = Subdivisao(NIVEL_SUBDIV, Ret1, Ret2, f1, f2, f3, g1, g2, g3);

  if (n_subdiv == 0) {
    Erro("Nenhum ponto encontrado na subdivisao");
    return 1;
  } else {
    if (MOSTRA_HIST)
      printf("\n\n       Pontos encontrados na subdivisao: %d (%s)",
				     (int) n_subdiv, DIV_NOME_ARQ);
    if (SALVA_HIST)
      fprintf(arq_execucao, "\n\n       Pontos encontrados na subdivisao: %d (%s)",
					     (int) n_subdiv, DIV_NOME_ARQ);
  }
    if (!FragmentaArquivoSubdiv(DIV_NOME_ARQ, NIVEL_CLASS)) return 1;
    n_pts_selec1 = SelecionaPontos(DIV_CLASS1, NIVEL_CLASS, 1);
    if (DOMINIO2) n_pts_selec2 = SelecionaPontos(DIV_CLASS2, NIVEL_CLASS, 2);

    if (MOSTRA_HIST)
      if (DOMINIO2) printf("\n       Pontos selecionados: %d (%s, %s)",
		(int) (n_pts_selec1 + n_pts_selec2), DIV_CLASS1, DIV_CLASS2);
      else printf("\n       Pontos selecionados: %d (%s)",
		(int) (n_pts_selec1 + n_pts_selec2), DIV_CLASS1);
    if (SALVA_HIST)
      if (DOMINIO2) fprintf(arq_execucao, "\n       Pontos selecionados: %d (%s, %s)",
		(int) (n_pts_selec1 + n_pts_selec2), DIV_CLASS1, DIV_CLASS2);
      else fprintf(arq_execucao, "\n       Pontos selecionados: %d (%s)",
		(int) (n_pts_selec1 + n_pts_selec2), DIV_CLASS1);

    #ifndef UNIX
    HoraAtual(&fim);                       /* Fim da subdivisao */
    tempo_gasto = DifTempo(inicio, fim);
    CriaArquivoTempoGasto(inicio, fim, "*** TEMPO GASTO NA SUBDIVISAO ***",
                                                               ARQ_TEMPO);
    if (MOSTRA_HIST)
      printf("\n\n# Tempo gasto na subdivisao: %g s", tempo_gasto);
    if (SALVA_HIST)
      fprintf(arq_execucao, "\n\n# Tempo gasto na subdivisao: %g s", tempo_gasto);
    #endif
  }

  /* ------------------------- Caminhada  -------------------------------- */

  if (MOSTRA_HIST)
    printf("\n\n# Caminhando com passo %s (L = %g)",
			    (CIRCULAR) ? "circular" : "tangencial", PASSO_L);
  if (SALVA_HIST)
    fprintf(arq_execucao, "\n\n# Caminhando com passo %s (L = %g)",
			    (CIRCULAR) ? "circular" : "tangencial", PASSO_L);

  #ifndef UNIX
  HoraAtual(&inicio);        /* Inicio da caminhada */
  #endif

  inicio_da_caminhada:

  n_pt++;
  if (n_pt <= n_pts_selec1) {
    LePontoSubdivDominio(DIV_CLASS1, Ret1, Ret2, &p0, &q0, pt_atual, n_pt);
  } else if (DOMINIO2 && n_pt - n_pts_selec1 <= n_pts_selec2) {
    LePontoSubdivDominio(DIV_CLASS2, Ret1, Ret2, &p0, &q0, pt_atual,
					   n_pt - n_pts_selec1);
  } else goto fim_da_caminhada;

  calcular_P:
  P.x = (f1(p0.u, p0.v) + g1(q0.u, q0.v))/2;
  P.y = (f2(p0.u, p0.v) + g2(q0.u, q0.v))/2;
  P.z = (f3(p0.u, p0.v) + g3(q0.u, q0.v))/2;

  p0inv = p0, q0inv = q0, Pinv = P;

  iteracoes = ConvPontoIntersecao(&P, &T, &p0, &q0, f1, f2, f3, g1, g2, g3);
  if (iteracoes == 0) {
    if (ISOLADO)
      VerificaPontoIsolado(p0.u, p0.v, f1, f2, f3, q0.u, q0.v, g1, g2, g3,
							    I_RAIO, I_INCR);
    if (n_pt <= n_pts_selec1) {
      if (LeOutroPonto(&p0, &q0, pt_atual, 1, &outro_novo)) {
	goto calcular_P;
      }
    } else if (DOMINIO2 && n_pt - n_pts_selec1 <= n_pts_selec2) {
      if (LeOutroPonto(&p0, &q0, pt_atual, 2, &outro_novo)) {
	goto calcular_P;
      }
    }
    goto inicio_da_caminhada;
  } else outro_novo = 1;

  P1 = P, T1 = T;

  T1inv.x = -T.x, T1inv.y = -T.y, T1inv.z = -T.z;
  P1inv = P;

  ProximoPontoVetorTangente(&P, T, PASSO_L);
  iteracoes = ConvPontoIntersecao(&P, &T, &p0, &q0, f1, f2, f3, g1, g2, g3);
  if (iteracoes == 0) {
    if (ISOLADO)
      VerificaPontoIsolado(p0.u, p0.v, f1, f2, f3, q0.u, q0.v, g1, g2, g3,
							    I_RAIO, I_INCR);
    goto inicio_da_caminhada;
  }
  p0inv = p0, q0inv = q0;

  P2 = P, T2 = T;

  if (CIRCULAR)
    CaminhadaPassoCircular(n_PONTOS, p0, q0, P1, T1, P2, T2, PASSO_L,
		     f1, f2, f3, g1, g2, g3, SENTIDO_VETOR_TANGENTE);
  else
    CaminhadaPassoTangencial(n_PONTOS, p0, q0, P2, T2, PASSO_L, f1, f2, f3,
					     g1, g2, g3);

  /* Caminhada no sentido contrario */
  Pinv = P1inv;
  ProximoPontoVetorTangente(&Pinv, T1inv, PASSO_L);
  iteracoes = ConvPontoIntersecao(&Pinv, &Tinv, &p0inv, &q0inv,
					f1, f2, f3, g1, g2, g3);
  if (iteracoes == 0) {
    if (ISOLADO)
      VerificaPontoIsolado(p0.u, p0.v, f1, f2, f3, q0.u, q0.v, g1, g2, g3,
							    I_RAIO, I_INCR);
    goto inicio_da_caminhada;
  }

  if (INVERSO) {
    P2inv = Pinv;
    T2inv.x = -Tinv.x, T2inv.y = -Tinv.y, T2inv.z = -Tinv.z;

    if (CIRCULAR)
      CaminhadaPassoCircular(n_PONTOS, p0inv, q0inv, P1inv, T1inv, P2inv, T2inv,
			    PASSO_L, f1, f2, f3, g1, g2, g3, SENTIDO_CONTRARIO);
    else
      CaminhadaPassoTangencial(n_PONTOS, p0inv, q0inv, P2inv, T2inv,
					    PASSO_L, f1, f2, f3, g1, g2, g3);
  }

  goto inicio_da_caminhada;

  fim_da_caminhada:

  #ifndef UNIX
  HoraAtual(&fim);      /* Fim da caminhada */
  #endif

  /* ------------------------- Finalizacao ------------------------------- */

  fclose(arq_padrao);

  if (argc == 1) {
    ApagaArqTemp(NIVEL_CLASS, 1);
    if (DOMINIO2) ApagaArqTemp(NIVEL_CLASS, 2);
  }

  if (MOSTRA_HIST)
    printf("\n\n# Pontos salvos em %s", ARQ_DADOS_PADRAO);
  if (SALVA_HIST)
    fprintf(arq_execucao, "\n\n# Pontos salvos em %s", ARQ_DADOS_PADRAO);

  if (ITERACOES) {
    if (MOSTRA_HIST) printf("\n");
    if (SALVA_HIST) fprintf(arq_execucao, "\n");
    total_pontos = CalculaIteracoes(ARQ_DADOS_PADRAO);
    if (MOSTRA_HIST) printf("\n       Total de pontos: %lu", total_pontos);
    if (SALVA_HIST)
      fprintf(arq_execucao, "\n       Total de pontos: %lu", total_pontos);
  }
  if (MOSTRA_HIST)
    printf("\n\n# Pontos distintos salvos em %s", ARQ_DADOS_DISTINTOS);
  if (SALVA_HIST)
    fprintf(arq_execucao, "\n\n# Pontos distintos salvos em %s", ARQ_DADOS_DISTINTOS);

  Simplifica(ARQ_DADOS_PADRAO, ARQ_DADOS_DISTINTOS, DISTANCIA, METRICA);

  if (ITERACOES) CalculaIteracoes(ARQ_DADOS_DISTINTOS);

  #ifndef UNIX
  if (SOM) {
    sound(3000); delay(200);
    sound(4000); delay(1500);
    sound(5000); delay(100);
    nosound();
  }
  tempo_gasto = DifTempo(inicio, fim);
  CriaArquivoTempoGasto(inicio, fim, "*** TEMPO GASTO NA CAMINHADA ***",
                                                                 ARQ_TEMPO);
  if (MOSTRA_HIST)
    printf("\n\n# Tempo gasto na caminhada: %g s", tempo_gasto);
  if (SALVA_HIST)
    fprintf(arq_execucao, "\n\n# Tempo gasto na caminhada: %g s", tempo_gasto);
  #endif

  if (SalvaConfiguracao()) {
    if (MOSTRA_HIST)
      printf("\n\n# Configuracao salva em %s\n", CONFIG_TXT);
    if (SALVA_HIST)
      fprintf(arq_execucao, "\n\n# Configuracao salva em %s\n", CONFIG_TXT);
  }

  if (SALVA_HIST) fclose(arq_execucao);
  return 0;
}

/* ----------------------------------------------------------------------- */

/*** FIM DE INTER.C ***/
