/*
  []---------------------------------------------------------------------[]
  |                                 STR.C                                 |
  |                                                                       |
  |                  Arquivo para operacoes com strings                   |
  |                                                                       |
  |                 (C) 1995, Lenimar Nunes de Andrade                    |
  []---------------------------------------------------------------------[]
*/

#define VERDADEIRO 1
#define FALSO      0

/*
			     =============
			      OBSERVAOES
			     =============

     Em todas as funoes deste arquivo:

   - o primeiro caracter de um string X e' o caracter X[0].

   - o parametro de retorno de uma funcao, se for um string, precisa ser
     suficientemente grande para conter a resposta.  O  tamanho  exigido
     no parametro de retorno deve poder ser calculado usando-se a funcao
     strlen() ou a macro TAMANHO() definida a seguir.
*/

/* ------------------------------------------------------------------------ */

/* Calcula o tamanho do string s. E' o mesmo que tam = strlen(s).
   Ex.: TAMANHO("teste", tam)   ---> tam = 5                       */

#define TAMANHO(s, tam)       tam = 0; if (s[0]) while(s[++tam])

/* ------------------------------------------------------------------------ */

char *Minusculas(char *s) {

/* Converte o string s para letras minusculas */

  int i = 0;
  while(s[i]) {
    if (s[i] >= 'A' && s[i] <= 'Z') s[i] += 32;
    i++;
  }
  return s;
}

/* ------------------------------------------------------------------------ */

int PosUlt(char x, char *s) {

/* Retorna a ultima posicao do caracter x no string s. Retorna -1 se
   x nao pertencer a s.
   Ex.: PosUlt('a', "Matematica") = 9        */

  register int i, tam;

  TAMANHO(s, tam);
  for(i = tam; i >= 0; i--)
    if (s[i] == x) return i;
  return -1;
}

/* ------------------------------------------------------------------------ */

int Pos(char x, char *s) {

/* Retorna a primeira posicao do caracter x no string s. Retorna -1 se
   x nao pertencer a s.
   Ex.: Pos('a', "Matematica") = 1           */

  register int i;

  for(i = 0; s[i]; i++)
    if (s[i] == x) return i;
  return -1;
}

/* ------------------------------------------------------------------------ */

/* Elimina espacos em branco de um string.
   Ex.: ELIMINAESPACOS(" A B C ") = "ABC"       */

#define ELIMINAESPACOS(s) EliminaCaracter(s, ' '); EliminaCaracter(s, 9)

/* ------------------------------------------------------------------------ */

char *EliminaCaracter(char *x, char car_x) {

/* Elimina todas as ocorrencias do caracter dado do string x.
   Ex.: EliminaEspacos(".1..2..3.", '.') = "123"    */

  register int i, j = 0, comprimento;
  char *aux;

  TAMANHO(x, comprimento);
  if ((aux = (char *) malloc(sizeof(char)*(comprimento + 1))) == NULL)
    return x;

  for (i = 0; i < comprimento; i++)
    if (x[i] != car_x) aux[j++] = x[i];
  aux[j] = '\0';
  strcpy(x, aux);
  free(aux);
  return x;
}

/* ------------------------------------------------------------------------ */

char *Substr(char *s, int inicio, int fim) {

/* Retorna um substring de um string s.
   Ex.: Substr("Joao Pessoa", 5, 8) = "Pess"    */

  register int i, tam;

  if (inicio > fim) return NULL;
  if (inicio < 0) inicio = 0;
  TAMANHO(s, tam);
  if (fim >= tam) fim = tam - 1;

  i = inicio;
  while (i <= fim) s[i - inicio] = s[i++];
  s[i - inicio] = '\0';
  return s;
}

/* ------------------------------------------------------------------------ */

/* Compara os n primeiros caracteres dos strings s1 e s2. Retorna 1 se
   forem identicos ou 0 forem distintos. Nao  e' feita distincao entre
   letras maiusculas e as respectivas letras minusculas.
   Ex.: COMPSTRNI("teste", "teste2...", 5) = 1
	COMPSTRNI("teste", "TESTE2...", 5) = 1                  */

#define COMPSTRNI(s1, s2, n) (ComparaNI(s1, s2, n) < 0)

/* ------------------------------------------------------------------------ */

int ComparaNI(char *s1, char *s2, int n) {

/* Compara os n primeiros caracteres dos strings  s1  e  s2 e retorna a
   posicao dos primeiros caracteres que forem diferentes. Retorna -1 se
   os strings coincidirem nos n primeiros caracteres. Letras maiusculas
   sao consideradas identicas `as respectivas letras minusculas.
   Ex.: ComparaNI("um", "dois", 10) = 0
	ComparaNI("teste", "TESTE", 4) = -1
	ComparaNI("Alo", "Alo!!!", 5) = 3
	ComparaNI("Alo", "Alo!!!", 3) = -1     */

  register int i = 0;

  while (i < n && s1[i] != '\0' && s2[i] != '\0' &&
				    toupper(s1[i]) == toupper(s2[i])) i++;
  if (i == n || (s1[i] == '\0' && s2[i] == '\0')) return -1;
  else return i;
}

/* ------------------------------------------------------------------------ */

void NomeArquivoCfg(char *nome_basico, char *extensao, char *nome_arq) {

  int tam, barra, ponto;

  barra = PosUlt('\\', nome_basico);
  tam = strlen(nome_basico);
  Substr(nome_basico, barra + 1, tam);
  ponto = Pos('.', nome_basico);
  Substr(nome_basico, 0, ponto - 1);
  sprintf(nome_arq, "%s.%s", nome_basico, extensao);
}

/* ------------------------------------------------------------------------- */

int InterpretaComando(char *comando) {

  char com_aux[80], result[80];
  int virgula, tam_str, i = 0;

  virgula = Pos(';', comando);
  if (virgula > 0) Substr(comando, 0, virgula - 1);
  ELIMINAESPACOS(comando);
  if (comando[0] == '.' || comando[0] == '%') return 1;
  TAMANHO(comando, tam_str);
  strcpy(com_aux, comando);
  strcpy(result, comando);
  while (comando[i] != '\0' && comando[i] != '=') i++;
  if (comando[i] == '=') {
    Substr(com_aux, 0, i - 1);
    Substr(result, i + 1, tam_str);
  } else return 1;

  if (COMPSTRNI(com_aux, "arq_dados", 9)) {
    strcpy(ARQ_DADOS_PADRAO, result);
  }

  else if (COMPSTRNI(com_aux, "xcentro", 7)) {
    X_CENTRO = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "ycentro", 7)) {
    Y_CENTRO = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "fi", 2)) {
    FI = atof(result);
  }

  else if (COMPSTRNI(com_aux, "teta", 4)) {
    TETA = atof(result);
  }

  else if (COMPSTRNI(com_aux, "passo", 5)) {
    PASSO_L = atof(result);
  }

  else if (COMPSTRNI(com_aux, "nivel_sub", 9)) {
    NIVEL_SUBDIV = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "nivel_cla", 9)) {
    NIVEL_CLASS = atoi(result);
  }


  else if (COMPSTRNI(com_aux, "dom2", 4)) {
    if (toupper(result[0]) == 'S' || result[0] == '1')
      DOMINIO2 = 1;
    else if (toupper(result[0]) == 'N' || result[0] == '0')
      DOMINIO2 = 0;
  }

  else if (COMPSTRNI(com_aux, "arq_erro", 8)) {
    strcpy(NOME_ARQ_ERROS, result);
  }

  else if (COMPSTRNI(com_aux, "subdiv1", 7)) {
    N_SUBDIV1 = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "subdiv2", 7)) {
    N_SUBDIV2 = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "subdiv3", 7)) {
    N_SUBDIV3 = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "subdiv4", 7)) {
    N_SUBDIV4 = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "subdiv5", 7)) {
    N_SUBDIV5 = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "ptdominio", 8)) {
    if (toupper(result[0]) == 'S' || result[0] == '1')
      DESENHAR_PONTOS_DOM = 1;
    else if (toupper(result[0]) == 'N' || result[0] == '0')
      DESENHAR_PONTOS_DOM = 0;
  }

  else if (COMPSTRNI(com_aux, "grade", 5)) {
    if (toupper(result[0]) == 'S' || result[0] == '1')
      GRADE = 1;
    else if (toupper(result[0]) == 'N' || result[0] == '0')
      GRADE = 0;
  }

  else if (COMPSTRNI(com_aux, "rastrear", 8)) {
    if (toupper(result[0]) == 'S' || result[0] == '1')
      DIV_RASTREAR = 1;
    else if (toupper(result[0]) == 'N' || result[0] == '0')
      DIV_RASTREAR = 0;
  }

  else if (COMPSTRNI(com_aux, "retorno", 7)) {
    if (toupper(result[0]) == 'S' || result[0] == '1')
      VERIFICA_RETORNO = 1;
    else if (toupper(result[0]) == 'N' || result[0] == '0')
      VERIFICA_RETORNO = 0;
  }

  else if (COMPSTRNI(com_aux, "circular", 8)) {
    if (toupper(result[0]) == 'S' || result[0] == '1')
      CIRCULAR = 1;
    else if (toupper(result[0]) == 'N' || result[0] == '0')
      CIRCULAR = 0;
  }

  else if (COMPSTRNI(com_aux, "div_dominio", 11)) {
    if (toupper(result[0]) == 'S' || result[0] == '1')
      DIV_ARQ_DOMINIO = 1;
    else if (toupper(result[0]) == 'N' || result[0] == '0')
      DIV_ARQ_DOMINIO = 0;
  }

  else if (COMPSTRNI(com_aux, "div_valores", 11)) {
    if (toupper(result[0]) == 'S' || result[0] == '1')
      DIV_ARQ_VALORES = 1;
    else if (toupper(result[0]) == 'N' || result[0] == '0')
      DIV_ARQ_VALORES = 0;
  }

  else if (COMPSTRNI(com_aux, "arq_subdiv", 10)) {
    strcpy(DIV_NOME_ARQ, result);
  }

  else if (COMPSTRNI(com_aux, "arq_class1", 10)) {
    strcpy(DIV_CLASS1, result);
  }

  else if (COMPSTRNI(com_aux, "arq_class2", 10)) {
    strcpy(DIV_CLASS2, result);
  }

  else if (COMPSTRNI(com_aux, "uminf", 5)) {
    Ret1.xmin = atof(result);
  }

  else if (COMPSTRNI(com_aux, "umaxf", 5)) {
    Ret1.xmax = atof(result);
  }

  else if (COMPSTRNI(com_aux, "vminf", 5)) {
    Ret1.ymin = atof(result);
  }

  else if (COMPSTRNI(com_aux, "vmaxf", 5)) {
    Ret1.ymax = atof(result);
  }

  else if (COMPSTRNI(com_aux, "uming", 5)) {
    Ret2.xmin = atof(result);
  }

  else if (COMPSTRNI(com_aux, "umaxg", 5)) {
    Ret2.xmax = atof(result);
  }

  else if (COMPSTRNI(com_aux, "vming", 5)) {
    Ret2.ymin = atof(result);
  }

  else if (COMPSTRNI(com_aux, "vmaxg", 5)) {
    Ret2.ymax = atof(result);
  }

  else if (COMPSTRNI(com_aux, "distmin", 7)) {
    DISTANCIA = atof(result);
  }

  else if (COMPSTRNI(com_aux, "x_subdiv", 8)) {
    if (toupper(result[0]) == 'S' || result[0] == '1')
      X_PONTOS_SUBDIVISAO = 1;
    else if (toupper(result[0]) == 'N' || result[0] == '0')
      X_PONTOS_SUBDIVISAO = 0;
  }

  else if (COMPSTRNI(com_aux, "dom_limit", 9)) {
    if (toupper(result[0]) == 'S' || result[0] == '1')
      DOMINIO_LIMITADO = 1;
    else if (toupper(result[0]) == 'N' || result[0] == '0')
      DOMINIO_LIMITADO = 0;
  }

  else if (COMPSTRNI(com_aux, "demora", 6)) {
    DEMORA = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "decimais", 8)) {
    ARQ_DEC = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "mens_dom_f", 10)) {
    strcpy(mens_Dom_F, result);
  }

  else if (COMPSTRNI(com_aux, "mens_dom_g", 10)) {
    strcpy(mens_Dom_G, result);
  }

  else if (COMPSTRNI(com_aux, "graf_x", 6)) {
    if (toupper(result[0]) == 'S' || result[0] == '1')
      GRAF_X = 1;
    else if (toupper(result[0]) == 'N' || result[0] == '0')
      GRAF_X = 0;
  }

  else if (COMPSTRNI(com_aux, "iteracoes", 8)) {
    if (toupper(result[0]) == 'S' || result[0] == '1')
      ITERACOES = 1;
    else if (toupper(result[0]) == 'N' || result[0] == '0')
      ITERACOES = 0;
  }

  else if (COMPSTRNI(com_aux, "arq_circ", 8)) {
    if (toupper(result[0]) == 'S' || result[0] == '1')
      ARQ_CIRC = 1;
    else if (toupper(result[0]) == 'N' || result[0] == '0')
      ARQ_CIRC = 0;
  }

  else if (COMPSTRNI(com_aux, "arq_dominio", 11)) {
    if (toupper(result[0]) == 'S' || result[0] == '1')
      ARQ_DOMINIO = 1;
    else if (toupper(result[0]) == 'N' || result[0] == '0')
      ARQ_DOMINIO = 0;
  }

  else if (COMPSTRNI(com_aux, "eixos", 5)) {
    if (toupper(result[0]) == 'S' || result[0] == '1')
      DES_EIXOS = 1;
    else if (toupper(result[0]) == 'N' || result[0] == '0')
      DES_EIXOS = 0;
  }

  else if (COMPSTRNI(com_aux, "isolado", 7)) {
    if (toupper(result[0]) == 'S' || result[0] == '1')
      ISOLADO = 1;
    else if (toupper(result[0]) == 'N' || result[0] == '0')
      ISOLADO = 0;
  }

  else if (COMPSTRNI(com_aux, "mouse", 5)) {
    if (toupper(result[0]) == 'S' || result[0] == '1')
      USAR_MOUSE = 1;
    else if (toupper(result[0]) == 'N' || result[0] == '0')
      USAR_MOUSE = 0;
  }

  else if (COMPSTRNI(com_aux, "arq_vetores", 9)) {
    if (toupper(result[0]) == 'S' || result[0] == '1')
      ARQ_VETORES = 1;
    else if (toupper(result[0]) == 'N' || result[0] == '0')
      ARQ_VETORES = 0;
  }

  else if (COMPSTRNI(com_aux, "salva_etapa", 11)) {
    if (toupper(result[0]) == 'S' || result[0] == '1')
      SALVA_HIST = 1;
    else if (toupper(result[0]) == 'N' || result[0] == '0')
      SALVA_HIST = 0;
  }

  else if (COMPSTRNI(com_aux, "mostra_etapa", 11)) {
    if (toupper(result[0]) == 'S' || result[0] == '1')
      MOSTRA_HIST = 1;
    else if (toupper(result[0]) == 'N' || result[0] == '0')
      MOSTRA_HIST = 0;
  }

  else if (COMPSTRNI(com_aux, "x_outros", 8)) {
    if (toupper(result[0]) == 'S' || result[0] == '1')
      X_OUTROS = 1;
    else if (toupper(result[0]) == 'N' || result[0] == '0')
      X_OUTROS = 0;
  }

  else if (COMPSTRNI(com_aux, "d_domf", 6)) {
    d_domF = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "c_domf", 6)) {
    c_domF = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "b_domf", 6)) {
    b_domF = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "a_domf", 6)) {
    a_domF = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "d_domg", 6)) {
    d_domG = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "c_domg", 6)) {
    c_domG = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "b_domg", 6)) {
    b_domG = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "a_domg", 6)) {
    a_domG = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "cor_domg", 8)) {
    cor_pontos_domG = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "cor_domf", 8)) {
    cor_pontos_domF = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "cor_moldg", 9)) {
    cor_molduraG = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "cor_moldf", 9)) {
    cor_molduraF = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "cor_inter", 9)) {
    COR_INTER = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "cor_eixos", 9)) {
    COR_EIXOS = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "tam_eixos", 9)) {
    TAM_EIXOS = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "cor_graf1", 9)) {
    COR_GRAF1 = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "cor_graf2", 9)) {
    COR_GRAF2 = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "tam", 3)) {
    TAM = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "cor_fundo", 9)) {
    COR_FUNDO = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "n_pontos", 8)) {
    n_PONTOS = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "nuf", 3)) {
    nuF = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "nug", 3)) {
    nuG = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "nvf", 3)) {
    nvF = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "nvg", 3)) {
    nvG = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "xcor", 4)) {
    X_COR_DOMINIO = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "corx3D", 6)) {
    corX3D = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "corx", 4)) {
    corX = atoi(result);
  }

  else if (COMPSTRNI(com_aux, "arq_config", 10)) {
    strcpy(CONFIG_TXT, result);
  }

  else if (COMPSTRNI(com_aux, "h_erro", 6)) {
    h_erro = atof(result);
  }

  else if (COMPSTRNI(com_aux, "z_erro", 6)) {
    z_erro = atof(result);
  }

  else if (COMPSTRNI(com_aux, "r_erro", 6)) {
    r_erro = atof(result);
  }

  else if (COMPSTRNI(com_aux, "emitir_som", 10)) {
    if (toupper(result[0]) == 'S' || result[0] == '1')
      SOM = 1;
    else if (toupper(result[0]) == 'N' || result[0] == '0')
      SOM = 0;
  }

  else if (COMPSTRNI(com_aux, "pontilhado", 8)) {
    if (toupper(result[0]) == 'S' || result[0] == '1')
      LINHA_PONTILHADA = 1;
    else if (toupper(result[0]) == 'N' || result[0] == '0')
      LINHA_PONTILHADA = 0;
  }
  else if (COMPSTRNI(com_aux, "inverso", 6)) {
    if (toupper(result[0]) == 'S' || result[0] == '1')
      INVERSO = 1;
    else if (toupper(result[0]) == 'N' || result[0] == '0')
      INVERSO = 0;
  }

  return 1;
}

/* ------------------------------------------------------------------------- */

int LeConfiguracao(char *nome_arq) {

  FILE *arq_cfg;
  char linha[200], *ch;

  arq_cfg = fopen(nome_arq, "rb");
  if (arq_cfg == NULL) return 0;

  do {
    ch = fgets(linha, 200, arq_cfg);
    if (ch != NULL) if (!InterpretaComando(linha)) return 0;
  } while (ch != NULL);

  fclose(arq_cfg);

  return 1;
}

/* ------------------------------------------------------------------------- */

char Sim[4] = "sim", Nao[4] = "nao";

char *sn(int x) {

/* Converte 0, 1 em "nao", "sim" */

  if (x == 0) return Nao;
  return Sim;
}

/* ------------------------------------------------------------------------- */

int SalvaConfiguracao(void) {

  FILE *arq;

  arq = fopen(CONFIG_TXT, "wt");
  if (arq == NULL) return 0;
  fprintf(arq,
  ". CONFIGURACAO DO PROGRAMA DE CALCULO DA INTERSECAO DE DUAS SUPERFICIES");
  fprintf(arq,
  "\n.......................................................................");
  fprintf(arq,
  "\n.\n. Constantes usadas em todo o programa\n.");
  fprintf(arq,
  "\narq_dados = %s   ; nome do arquivo de dados padrao", ARQ_DADOS_PADRAO);
  fprintf(arq,
  "\nmouse = %s             ; determina se usa mouse nos pts dominio",
   sn(USAR_MOUSE));
  fprintf(arq,
  "\ncircular = %s          ; determina se usa metodo do passo circular",
    sn(CIRCULAR));

  fprintf(arq,
  "\npasso = %6.6f        ; passo L dos metodos de caminhada", PASSO_L);
  fprintf(arq,
  "\ndistmin = %6.6f      ; distancia minima entre os pontos encontrados", DISTANCIA);
  fprintf(arq,
  "\nh_erro = %10.8f     ; erro usado no calculo de derivadas", h_erro);
  fprintf(arq,
  "\nz_erro = %10.8f     ; erro na identificacao de dois numeros reais",z_erro);
  fprintf(arq,
  "\nr_erro = %10.8f     ; erro na verif. de retorno ao ponto inicial", r_erro);
  fprintf(arq,
  "\n.......................................................................");
  fprintf(arq,
  "\n.\n. Constantes usadas no metodo da subdivisao\n.");
  fprintf(arq,
  "\nnivel_sub =%2d           ; nivel maximo de subdivisao", NIVEL_SUBDIV);
  fprintf(arq,
  "\nnivel_cla =%2d           ; nivel de selecao de pontos subdiv.",NIVEL_CLASS);
  fprintf(arq,
  "\nsubdiv1 =%3d            ; usado no nivel 1 da subdivisao", N_SUBDIV1);
  fprintf(arq,
  "\nsubdiv2 =%3d            ; usado no nivel 2 da subdivisao", N_SUBDIV2);
  fprintf(arq,
  "\nsubdiv3 =%3d            ; usado no nivel 3 da subdivisao", N_SUBDIV3);
  fprintf(arq,
  "\nsubdiv4 =%3d            ; usado no nivel 4 da subdivisao", N_SUBDIV4);
  fprintf(arq,
  "\nsubdiv5 =%3d            ; usado no nivel 5 da subdivisao", N_SUBDIV5);
  fprintf(arq,
  "\nrastrear = %s          ; determina se mostra pontos encontrados",
   sn(DIV_RASTREAR));
  fprintf(arq,
  "\ndiv_dominio = %s       ; determina se salva em arquivo pts dominio",
   sn(DIV_ARQ_DOMINIO));
  fprintf(arq,
  "\ndiv_valores = %s       ; determina se salva valores dos pontos",
   sn(DIV_ARQ_VALORES));
  fprintf(arq,
  "\narq_subdiv = %s ; arquivo com pts encontrados na subdivisao",
   DIV_NOME_ARQ);
  fprintf(arq,
  "\narq_class1 = %s ; nome do arquivo 1 de pts selecionados", DIV_CLASS1);
  fprintf(arq,
  "\narq_class2 = %s ; nome do arquivo 2 de pts selecionados", DIV_CLASS2);
  fprintf(arq,
  "\nx_outros = %s          ; determina se marca outros pontos nos dominios",
   sn(X_OUTROS));
  fprintf(arq,
  "\ndom2 = %s              ; determina se seleciona pontos em dom(G)",
   sn(DOMINIO2));
  fprintf(arq,
  "\n.......................................................................");
  fprintf(arq,
  "\n.\n. Constantes usadas na construcao dos graficos das superficies\n.");
  fprintf(arq,
  "\ntam =%4d               ; constante de ampliacao do grafico", TAM);
  fprintf(arq,
  "\nxcentro =%4d           ; abscissa do centro da tela", X_CENTRO);
  fprintf(arq,
  "\nycentro =%4d           ; ordenada do centro da tela", Y_CENTRO);
  fprintf(arq, "\nfi =%9.6f           ; angulo de rotacao (eixo z)", FI);
  fprintf(arq, "\nteta =%9.6f         ; angulo de rotacao (eixo x)", TETA);
  fprintf(arq,
  "\numinf =%9.6f        ; valor minimo do parametro u (superf. F)", Ret1.xmin);
  fprintf(arq,
  "\numaxf =%9.6f        ; valor maximo do parametro u (superf. F)", Ret1.xmax);
  fprintf(arq,
  "\nvminf =%9.6f        ; valor minimo do parametro v (superf. F)", Ret1.ymin);
  fprintf(arq,
  "\nvmaxf =%9.6f        ; valor maximo do parametro v (superf. F)", Ret1.ymax);
  fprintf(arq,
  "\numing =%9.6f        ; valor minimo do parametro u (superf. G)", Ret2.xmin);
  fprintf(arq,
  "\numaxg =%9.6f        ; valor maximo do parametro u (superf. G)", Ret2.xmax);
  fprintf(arq,
  "\nvming =%9.6f        ; valor minimo do parametro v (superf. G)", Ret2.ymin);
  fprintf(arq,
  "\nvmaxg =%9.6f        ; valor maximo do parametro v (superf. G)", Ret2.ymax);
  fprintf(arq,
  "\nnuf =%3d                ; numero de subdivisoes de [uminf, umaxf]", nuF);
  fprintf(arq,
  "\nnvf =%3d                ; numero de subdivisoes de [vminf, vmaxf]", nvF);
  fprintf(arq,
  "\nnug =%3d                ; numero de subdivisoes de [uming, umaxg]", nuG);
  fprintf(arq,
  "\nnvg =%3d                ; numero de subdivisoes de [vming, vmaxg]", nvG);
  fprintf(arq,
  "\ncor_graf1 =%3d          ; cor do grafico da superficie F", COR_GRAF1);
  fprintf(arq,
  "\ncor_graf2 =%3d          ; cor do grafico da superficie G", COR_GRAF2);
  fprintf(arq,
  "\ncor_fundo =%3d          ; cor de fundo", COR_FUNDO);
  fprintf(arq,
  "\ncor_inter =%3d          ; cor da curva-intersecao", COR_INTER);
  fprintf(arq,
  "\ncor_eixos =%3d          ; cor dos eixos", COR_EIXOS);
  fprintf(arq,
  "\ntam_eixos =%3d          ; tamanho dos eixos", TAM_EIXOS);
  fprintf(arq,
  "\nn_pontos =%4d          ; numero maximo de pontos em cada trecho", n_PONTOS);
  fprintf(arq,
  "\ngraf_x = %s            ; determina se marca pts intersecao com um \"x\"", sn(GRAF_X));
  fprintf(arq,
  "\neixos = %s             ; determina se mostra eixos x, y, z", sn(DES_EIXOS));
  fprintf(arq,
  "\npontilhado = %s        ; determina se usa linha pontilhada em graf. F, G",
							 sn(LINHA_PONTILHADA));
  fprintf(arq,
  "\n.......................................................................");
  fprintf(arq,
  "\n.\n. Constantes usadas na construcao dos graficos dos dominios\n.");
  fprintf(arq,
  "\nmens_dom_f= %s; mensagem mostrada no dominio de F", mens_Dom_F);
  fprintf(arq,
  "\na_domf =%4d            ; (a_domf, b_domf) -- (c_domf, d_domf)", a_domF);
  fprintf(arq,
  "\nb_domf =%4d            ; determinam a posicao do dom(F) na tela", b_domF);
  fprintf(arq,
  "\nc_domf =%4d", c_domF);
  fprintf(arq,
  "\nd_domf =%4d", d_domF);
  fprintf(arq,
  "\ncor_domf =%3d           ; cor dos pontos no dominio de F", cor_pontos_domF);
  fprintf(arq,
  "\ncor_moldf =%3d          ; cor da moldura do dominio de F", cor_molduraF);
  fprintf(arq,
  "\nmens_dom_g= %s; mensagem mostrada no dominio de G", mens_Dom_G);
  fprintf(arq,
  "\na_domg =%4d            ; (a_domg, b_domg) -- (c_domg, d_domg)", a_domG);
  fprintf(arq,
  "\nb_domg =%4d            ; determinam a posicao do dom(G) na tela", b_domG);
  fprintf(arq,
  "\nc_domg =%4d", c_domG);
  fprintf(arq,
  "\nd_domg =%4d", d_domG);
  fprintf(arq,
  "\ncor_domg =%3d           ; cor dos pontos no dominio de G", cor_pontos_domG);
  fprintf(arq,
  "\ncor_moldg =%3d          ; cor da moldura do dominio de G", cor_molduraG);
  fprintf(arq,
  "\nxcor =%3d               ; cor dos pontos obtidos na subdivisao",
   X_COR_DOMINIO);
  fprintf(arq,
  "\ncorx =%3d               ; cor de pts dominio selecionados com o mouse",
   corX);
  fprintf(arq,
  "\ncorx3d =%3d             ; cor de pts espaco selecionados com o mouse",
   corX3D);
  fprintf(arq,
  "\nx_subdiv = %s          ; determina se marca com \"x\" pts subdiv.",
   sn(X_PONTOS_SUBDIVISAO));
  fprintf(arq,
  "\ndom_limit = %s         ; determina se considera dominio limitado",
    sn(DOMINIO_LIMITADO));
  fprintf(arq,
  "\nisolado = %s           ; determina se calcula pontos isolados",
    sn(ISOLADO));
  fprintf(arq,
  "\n.......................................................................");
  fprintf(arq, "\n.\n. Outras constantes\n.");
  fprintf(arq,
  "\narq_config = %s ; arquivo onde e' salva a configuracao", CONFIG_TXT);
  fprintf(arq,
  "\narq_erro = %s    ; nome do arquivo das mensagens de erro",NOME_ARQ_ERROS);
  fprintf(arq,
  "\ndemora =%4d            ; milisegundos entre dois pontos mostrados",
    DEMORA);
  fprintf(arq,
  "\narq_vetores = %s       ; determina se salva vetores no arquivo",
    sn(ARQ_VETORES));
  fprintf(arq,
  "\ndecimais =%3d           ; casas decimais utilizadas em valores salvos",
    ARQ_DEC);
  fprintf(arq,
  "\niteracoes = %s         ; determina se salva iteracoes em arquivo",
    sn(ITERACOES));
  fprintf(arq,
  "\nemitir_som = %s        ; determina se emite som no final da execucao",
    sn(SOM));
  fprintf(arq,
  "\narq_circ = %s          ; determina se salva centros dos circulos",
    sn(ARQ_CIRC));
  fprintf(arq,
  "\narq_dominio = %s       ; determina se salva pontos do dominio",
    sn(ARQ_DOMINIO));
  fprintf(arq,
  "\nmostra_etapas = %s     ; determina se mostra etapas da execucao",
    sn(MOSTRA_HIST));
  fprintf(arq,
  "\nsalva_etapas = %s      ; determina se salva etapas da execucao",
    sn(SALVA_HIST));
  fprintf(arq,
  "\ninicial = %s           ; determina se marca ponto inicial/final",
    sn(INICIAL));
  fprintf(arq,
  "\ninverso = %s           ; determina se percorre no sentido contrario",
    sn(INVERSO));
  fprintf(arq,
  "\nretorno = %s           ; determina se verifica volta ao ponto inicial",
    sn(VERIFICA_RETORNO));
  fprintf(arq,
  "\nptdominio = %s         ; determina se desenha pontos nos dominios",
    sn(DESENHAR_PONTOS_DOM));
  fprintf(arq,
  "\ngrade = %s             ; determina se desenha grade nos dominios",
    sn(GRADE));

  fprintf(arq,
  "\n......................................................................");
  fprintf(arq, "\n.");
  fprintf(arq,
  "\n. Informacoes sobre o arquivo de configuracao:");
  fprintf(arq, "\n.");
  fprintf(arq,
  "\n% A linha de comentario pode iniciar com um ponto ou com um sinal de %");
  fprintf(arq, "\n.");
  fprintf(arq,
  "\n. Comentarios tambem podem ser escritos `a direita de ponto e virgula");
  fprintf(arq, "\n.");
  fprintf(arq,
  "\n. Para algumas variaveis o \"sim\" pode ser fornecido como um \"1\" ou co-");
  fprintf(arq,
  "\n. mo um \"s\". O \"nao\" pode ser fornecido como um \"0\" ou como um \"n\".");
  fprintf(arq, "\n.");
  fprintf(arq,
  "\n. Nao e' feita distincao entre letras maiusculas ou minusculas.");

  #ifndef UNIX
  fprintf(arq, "\n.");
  fprintf(arq,
  "\n. O arquivo de  configuracao deve ter  o mesmo nome que o arquivo exe-");
  fprintf(arq,
  "\n. cutavel, mas com extensao CFG. (Por exemplo,  PROG.EXE deve ter con-");
  fprintf(arq, "\n. figuracao PROG.CFG)");
  fprintf(arq, "\n.");
  fprintf(arq, "\n. Tabela de cores:\n.");
  fprintf(arq,
  "\n.     0 = BLACK        1 = BLUE            2 = GREEN");
  fprintf(arq,
  "\n.     3 = CYAN         4 = RED             5 = MAGENTA");
  fprintf(arq,
  "\n.     6 = BROWN        7 = LIGHTGRAY       8 = DARKGRAY");
  fprintf(arq,
  "\n.     9 = LIGHTBLUE   10 = LIGHTGREEN     11 = LIGHTCYAN");
  fprintf(arq,
  "\n.    12 = LIGHTRED    13 = LIGHTMAGENTA   14 = YELLOW");
  fprintf(arq,
  "\n.    15 = WHITE");
  #endif

  fprintf(arq, "\n.");
  fprintf(arq, "\n...FIM...");

  fclose(arq);
  return 1;
}

/* ------------------------------------------------------------------------- */

/*** FIM DE STR.C ***/
